﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;

using System.Windows.Forms;
using System.Drawing.Drawing2D;

using System.Windows.Automation;
using System.Windows.Automation.Provider;
using System.Security.Permissions;

namespace ClockUserControl
{
    public partial class UserControlClock : UserControl, IRawElementProviderSimple
    {
       DateTime Time = DateTime.Now;
       public UserControlClock()
        {
            InitializeComponent();
            myHandle = Handle;
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            this.Time = DateTime.Now;
            Refresh();            
        }

        #region draw clock
        private void UserClock_Paint(object sender, PaintEventArgs e)
        {
            Graphics dc = e.Graphics;
            Pen pn = new Pen(ForeColor);
            SolidBrush br = new SolidBrush(ForeColor);
            initCoordinates(dc);  
            DrawDots(dc, br);
            DrawHourHand(dc, pn);
            DrawSecondHand(dc, pn);
            DrawMinuteHand(dc, pn);
        }
        
        public void initCoordinates(Graphics dc)
        {
            if (this.Width == 0 || this.Height == 0) return;
            dc.TranslateTransform(this.Width / 2, this.Height / 2);
            dc.ScaleTransform(this.Height / 250F, this.Width / 250F);
        }
        public void DrawDots(Graphics dc, Brush brush)
        {
            int iSize;
            for (int i = 0; i <= 59; i++)
            {
                if (i % 5 == 0)
                {
                    iSize = 15;
                }
                else
                {
                    iSize = 5;
                }
                dc.FillEllipse(brush, -iSize / 2, -100 - iSize / 2, iSize, iSize);
                dc.RotateTransform(6);
            }
        }
        public virtual void DrawHourHand(Graphics grfx, Pen pn)
        {
            GraphicsState gs = grfx.Save();
            grfx.RotateTransform(360.0F * Time.Hour / 12 + 30.0F * Time.Minute / 60);
            grfx.DrawLine(pn, 0, 0, 0, -50);
            grfx.Restore(gs);
        }
        public virtual void DrawMinuteHand(Graphics grfx, Pen pn)
        {
            GraphicsState gs = grfx.Save();
            grfx.RotateTransform(360.0F * Time.Minute / 60 + 6.0F * Time.Second / 60);
            grfx.DrawLine(pn, 0, 0, 0, -70);
            grfx.Restore(gs);
        }
        public virtual void DrawSecondHand(Graphics grfx, Pen pn)
        {
            GraphicsState gs = grfx.Save();
            grfx.RotateTransform(360.0F * Time.Second / 60);
            grfx.DrawLine(pn, 0, 0, 0, -100);
            grfx.Restore(gs);
        }
        #endregion

        #region SeverSider UIA Provider
        IntPtr myHandle;
        
        [PermissionSetAttribute(SecurityAction.Demand, Unrestricted = true)]
        protected override void WndProc(ref Message m)
        {
            const int WM_GETOBJECT = 0x003D;

            if ((m.Msg == WM_GETOBJECT) && (m.LParam.ToInt32() == AutomationInteropProvider.RootObjectId))
            {
                m.Result = AutomationInteropProvider.ReturnRawElementProvider(
                    Handle, m.WParam, m.LParam, (IRawElementProviderSimple)this);
                return;
            }
            base.WndProc(ref m);
        }
        
        object IRawElementProviderSimple.GetPropertyValue(int propId)
        {
            if (propId == AutomationElementIdentifiers.ClassNameProperty.Id)
            {
                return "ClockUserControlClass";
            }
            else if (propId == AutomationElementIdentifiers.ControlTypeProperty.Id)
            {
                return ControlType.Custom.Id;
            }
            if (propId == AutomationElementIdentifiers.HelpTextProperty.Id)
            {
                return "Show time in Clock.";
            }
            if (propId == AutomationElementIdentifiers.IsEnabledProperty.Id)
            {
                return true;
            }
            else
            {
                return null;
            }
        }

       object IRawElementProviderSimple.GetPatternProvider(int patternId)
        {  
                return null;
            
        }
        ProviderOptions IRawElementProviderSimple.ProviderOptions
         {
             get
             {
                 return ProviderOptions.ServerSideProvider;
             }
         }
        IRawElementProviderSimple IRawElementProviderSimple.HostRawElementProvider
        {
            get
            {
                return AutomationInteropProvider.HostProviderFromHandle(myHandle);
            }
        }

        #endregion
    }
}
